/*CSS edits to more closely match github heading styles*/
h1, h2 {
    border-radius: 0;
    padding-bottom: 0.3em;
    border-bottom: 1px solid var(--color-foreground-border);
}

h1 {
    padding-bottom: 0.3em;
    font-size: 2.0em;
}

h2 {
    padding-bottom: 0.3em;
    font-size: 1.5em;
}

h3 {
    font-size: 1.25em;
    margin: 1em 0;
    padding-left: 0;
}

h4, h5, h6 {
    font-size: 1.0em;
}

/* Format sequential definition lists to display together--no extra spaces */
dl {
    margin-block-start: 0;
    margin-block-end: 0;
}

dd {
    margin-block-start: 0.25em;
}
